<?php
/**
 * @package        Joomla
 * @subpackage     Membership Pro
 * @author         Tuan Pham Ngoc
 * @copyright      Copyright (C) 2012 - 2021 Ossolution Team
 * @license        GNU/GPL, see LICENSE.php
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Mail\MailHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Table\Table;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;

class plgOSMembershipNSPro extends CMSPlugin
{
	/**
	 * Application object.
	 *
	 * @var    JApplicationCms
	 */
	protected $app;

	/**
	 * Database object.
	 *
	 * @var    JDatabaseDriver
	 */
	protected $db;

	/**
	 * Make language files will be loaded automatically.
	 *
	 * @var bool
	 */
	protected $autoloadLanguage = true;

	/**
	 * Constructor
	 *
	 * @param   object &$subject  The object to observe
	 * @param   array   $config   An optional associative array of configuration settings.
	 */
	public function __construct($subject, array $config = [])
	{
		if (!file_exists(JPATH_ADMINISTRATOR . '/components/com_nspro/lib.php'))
		{
			return;
		}

		parent::__construct($subject, $config);
	}

	/**
	 * Method to check if the plugin is executable
	 *
	 * @return bool
	 */
	private function isExecutable()
	{
	    if (!$this->app)
        {
            return false;
        }

	    if ($this->app->isClient('site') && !$this->params->get('show_on_frontend'))
		{
			return false;
		}

		return true;
	}

	/**
	 * Display form allows users to change settings on subscription plan add/edit screen
	 *
	 * @param   OSMembershipTablePlan  $row
	 */
	private function drawSettingForm($row)
	{
		$params               = new Registry($row->params);
		$activeAssignListIds  = explode(',', $params->get('nspro_activated_assign_ids', ''));
		$activeRemoveListIds  = explode(',', $params->get('nspro_activated_remove_ids', ''));
		$expiredRemoveListIds = explode(',', $params->get('nspro_expired_remove_ids', ''));
		$expiredAssignListIds = explode(',', $params->get('nspro_expired_assign_ids', ''));

		$lists = $this->db->setQuery('SELECT `id`, `lname` FROM `#__nspro_lists`')->loadObjectList();

		$query = $this->db->getQuery(true);
		$query->select('id, name')->from('#__osmembership_fields')->where('published = 1')->where('fieldtype = "Checkboxes"')->order('name');
		$this->db->setQuery($query);
		$mailingListFields = $this->db->loadObjectList();
		?>
        <div class="row-fluid">
            <div class="span6 pull-left">
                <fieldset class="adminform">
                    <legend><?php echo Text::_('OSM_WHEN_SUBSCRIPTION_ACTIVE'); ?></legend>
                    <div class="control-group">
                        <div class="control-label">
	                        <?php echo OSMembershipHelperHtml::getFieldLabel('nspro_activated_assign_ids', Text::_('OSM_ASSIGN_TO_MAILING_LISTS'), Text::_('OSM_ASSIGN_TO_MAILING_LISTS_EXPLAIN')); ?>
                        </div>
                        <div class="controls">
	                        <?php echo OSMembershipHelperHtml::getChoicesJsSelect(HTMLHelper::_('select.genericlist', $lists, 'nspro_activated_assign_ids[]', 'class="form-select advSelect" multiple="multiple" size="10"', 'id', 'lname', $activeAssignListIds)); ?>
                        </div>
                    </div>
                    <div class="control-group">
                        <div class="control-label">
	                        <?php echo OSMembershipHelperHtml::getFieldLabel('nspro_activated_remove_ids', Text::_('OSM_REMOVE_FROM_MAILING_LISTS'), Text::_('OSM_REMOVE_FROM_MAILING_LISTS_EXPLAIN')); ?>
                        </div>
                        <div class="controls">
	                        <?php echo OSMembershipHelperHtml::getChoicesJsSelect(HTMLHelper::_('select.genericlist', $lists, 'nspro_activated_remove_ids[]', 'class="form-select advSelect" multiple="multiple" size="10"', 'id', 'lname', $activeRemoveListIds)); ?>
                        </div>
                    </div>
					<?php
					if (count($mailingListFields))
					{
						$options   = [];
						$options[] = HTMLHelper::_('select.option', '', 'Select Field', 'id', 'name');
						$options   = array_merge($options, $mailingListFields);
						?>
                        <div class="control-group">
                            <div class="control-label">
								<?php echo OSMembershipHelperHtml::getFieldLabel('mailing_list_custom_field', Text::_('OSM_MAILING_LISTS_CUSTOM_FIELD'), Text::_('OSM_MAILING_LISTS_CUSTOM_FIELD_EXPLAIN')); ?>
                            </div>
                            <div class="controls">
	                            <?php echo OSMembershipHelperHtml::getChoicesJsSelect(HTMLHelper::_('select.genericlist', $options, 'mailing_list_custom_field', 'class="form-select advSelect"', 'id', 'name', (int) $params->get('mailing_list_custom_field'))); ?>
                            </div>
                        </div>
						<?php
					}
					?>
                </fieldset>
            </div>
            <div class="span6 pull-left">
                <fieldset class="adminform">
                    <legend><?php echo Text::_('OSM_WHEN_SUBSCRIPTION_EXPIRED'); ?></legend>
                    <div class="control-group">
                        <div class="control-label">
	                        <?php echo OSMembershipHelperHtml::getFieldLabel('nspro_expired_remove_ids', Text::_('OSM_REMOVE_FROM_MAILING_LISTS'), Text::_('OSM_REMOVE_FROM_MAILING_LISTS_EXPLAIN')); ?>
                        </div>
                        <div class="controls">
							<?php echo OSMembershipHelperHtml::getChoicesJsSelect(HTMLHelper::_('select.genericlist', $lists, 'nspro_expired_remove_ids[]', 'class="form-select advSelect" multiple="multiple" size="10"', 'id', 'lname', $expiredRemoveListIds)); ?>
                        </div>
                    </div>
                    <div class="control-group">
                        <div class="control-label">
							<?php echo OSMembershipHelperHtml::getFieldLabel('nspro_expired_assign_ids', Text::_('OSM_ASSIGN_TO_MAILING_LISTS'), Text::_('OSM_ASSIGN_TO_MAILING_LISTS_EXPLAIN')); ?>
                        </div>
                        <div class="controls">
							<?php echo OSMembershipHelperHtml::getChoicesJsSelect(HTMLHelper::_('select.genericlist', $lists, 'nspro_expired_assign_ids[]', 'class="form-select advSelect" multiple="multiple" size="10"', 'id', 'lname', $expiredAssignListIds)); ?>
                        </div>
                    </div>
            </div>
        </div>
		<?php
	}

	/**
	 * Return list of custom fields in NS Pro which will be used to map with fields in Events Booking
	 *
	 * @return array
	 */
	public function onGetNewsletterFields()
	{
	    if (!$this->app)
        {
            return [];
        }

	    $db    = $this->db;
		$query = $db->getQuery(true)->select($db->quoteName(['id', 'name'], ['value', 'text']))->from('#__nspro_fields');
		$db->setQuery($query);

		return $db->loadObjectList();
	}

	/**
	 * Render setting form
	 *
	 * @param   OSMembershipTablePlan  $row
	 *
	 * @return array
	 */
	public function onEditSubscriptionPlan($row)
	{
		if (!$this->isExecutable())
		{
			return [];
		}

		ob_start();
		$this->drawSettingForm($row);
		$form = ob_get_clean();

		return ['title' => Text::_('PLG_OSMEMBERSHIP_NSPRO_LIST_SETTINGS'), 'form'  => $form];
	}

	/**
	 * Store setting into database, in this case, use params field of plans table
	 *
	 * @param   OSMembershipTablePlan  $row
	 * @param   bool                   $isNew  true if create new plan, false if edit
	 *
	 * @return void
	 */
	public function onAfterSaveSubscriptionPlan($context, $row, $data, $isNew)
	{
		if (!$this->isExecutable())
		{
			return;
		}

		$params = new Registry($row->params);

		// Prevent PHP notices/warnings
		$keys = ['nspro_activated_assign_ids', 'nspro_activated_remove_ids', 'nspro_expired_remove_ids', 'nspro_expired_assign_ids'];

		foreach ($keys as $key)
		{
			if (!isset($data[$key]))
			{
				$data[$key] = [];
			}
		}

		$params->set('nspro_activated_assign_ids', implode(',', $data['nspro_activated_assign_ids']));
		$params->set('nspro_activated_remove_ids', implode(',', $data['nspro_activated_remove_ids']));
		$params->set('nspro_expired_remove_ids', implode(',', $data['nspro_expired_remove_ids']));
		$params->set('nspro_expired_assign_ids', implode(',', $data['nspro_expired_assign_ids']));
		$params->set('mailing_list_custom_field', isset($data['mailing_list_custom_field']) ? $data['mailing_list_custom_field'] : 0);
		$row->params = $params->toString();

		$row->store();
	}

	/**
	 * Run when a membership activated
	 *
	 * @param   OSMembershipTableSubscriber  $row
	 */
	public function onMembershipActive($row)
	{
		if (!$this->app)
		{
		    return;
		}

		if (!MailHelper::isEmailAddress($row->email))
		{
			return;
		}

		$config = OSMembershipHelper::getConfig();

		// In case subscriber doesn't want to subscribe to newsleter, stop
		if ($config->show_subscribe_newsletter_checkbox && empty($row->subscribe_newsletter))
		{
			return;
		}

		/* @var OSMembershipTablePlan $plan */
		$plan = Table::getInstance('Plan', 'OSMembershipTable');
		$plan->load($row->plan_id);
		$params = new Registry($plan->params);

		$db    = $this->db;
		$query = $db->getQuery(true);

		if ($fieldId = (int) $params->get('mailing_list_custom_field'))
		{
			$query->select('field_value')
				->from('#__osmembership_field_value')
				->where('subscriber_id = ' . $row->id)
				->where('field_id = ' . $fieldId);
			$db->setQuery($query);
			$fieldValue = $db->loadResult();

			if ($fieldValue && is_array(json_decode($fieldValue)))
			{
				$listNames = array_map('trim', json_decode($fieldValue));
			}
            elseif (is_string($fieldValue) && strpos($fieldValue, ', ') !== false)
			{
				$listNames = explode(', ', $fieldValue);
			}
            elseif (is_string($fieldValue))
			{
				$listNames = [$fieldValue];
			}
			else
			{
				$listNames = [];
			}

			if (!empty($listNames))
			{
				$listNames = array_map([$db, 'quote'], $listNames);

				$query->clear()
					->select($db->quoteName('id'))
					->from('#__nspro_lists')
					->where($db->quoteName('state') . ' = 1')
					->where('(lname = ' . implode(' OR lname = ', $listNames) . ')');
				$db->setQuery($query);
				$listIds = implode(',', $db->loadColumn());
			}
			else
			{
				$listIds = '';
			}
		}
		else
		{
			$listIds = trim($params->get('nspro_activated_assign_ids', ''));
		}

		$removeListIds = trim($params->get('nspro_activated_remove_ids'));

		if ($listIds || $removeListIds)
		{
			require_once(JPATH_ADMINISTRATOR.'/components/com_nspro/lib.php');

			$lib_result = add_or_update_subscriber(trim($row->first_name . ' ' . $row->last_name), $row->email, 'auto', '1', array());
			if ($lib_result === false) { // Something went wrong
				return;
			}

			$this->updateNSProFieldValues($row, $lib_result->insertid);

			if ($listIds)
			{
                DoAddListsToSubscriber($listIds, $lib_result->insertid);
			}

			if ($removeListIds)
			{
				DoRemoveListsFromSubscriber($removeListIds, $lib_result->insertid);
			}
		}
	}

	/**
	 * Plugin triggered when user update his profile
	 *
	 * @param   OSMembershipTableSubscriber  $row  The subscription record
	 */
	public function onProfileUpdate($row)
	{
		if (!$this->app)
		{
			return;
		}

		if (!MailHelper::isEmailAddress($row->email))
		{
			return;
		}

		$query = $this->db->getQuery(true);
		$user  = Factory::getUser($row->user_id);
		$query->update('#__nspro_subs')
			->set('email = ' . $this->db->quote($row->email))
			->where('email = ' . $this->db->quote($user->email));
		$this->db->setQuery($query);

		try
		{
			$this->db->execute();
		}
		catch (Exception $e)
		{
			// There is another subscriber with this email, ignore
			return;
		}

		$subId = $this->db->setQuery('SELECT `id` FROM `#__nspro_subs` WHERE `email` = "'.$this->db->escape($row->email).'"')->loadResult();
		if ($subId) {
			$this->updateNSProFieldValues($row, $subId);
		}
	}

	/**
	 * Run when a membership expiried die
	 *
	 * @param   OSMembershipTableSubscriber  $row
	 */
	public function onMembershipExpire($row)
	{
		if (!$this->app)
		{
			return;
		}

		if (!MailHelper::isEmailAddress($row->email))
		{
			return;
		}

		$config = OSMembershipHelper::getConfig();

		// In case subscriber doesn't want to subscribe to newsleter, stop
		if ($config->show_subscribe_newsletter_checkbox && empty($row->subscribe_newsletter))
		{
			return;
		}

		/* @var OSMembershipTablePlan $plan */
		$plan = Table::getInstance('Plan', 'OSMembershipTable');
		$plan->load($row->plan_id);
		$params        = new Registry($plan->params);
		$listIds       = trim($params->get('nspro_expired_remove_ids', ''));
		$assignListIds = trim($params->get('nspro_expired_assign_ids', ''));

		if ($row->user_id)
		{
			$activePlans = OSMembershipHelperSubscription::getActiveMembershipPlans($row->user_id, [$row->id]);

			// He renewed his subscription before, so don't remove him from the lists
			if (in_array($row->plan_id, $activePlans))
			{
				return;
			}
		}

		if ($listIds != '' || $assignListIds != '')
		{
			require_once(JPATH_ADMINISTRATOR.'/components/com_nspro/lib.php');

			$subId = $this->db->setQuery('SELECT `id` FROM `#__nspro_subs` WHERE `email` = "'.$this->db->escape($row->email).'"')->loadResult();

			if (!$subId && $assignListIds)
			{
				// Create new subscriber as it is needed to assign user to the lists
				$lib_result = add_or_update_subscriber(trim($row->first_name . ' ' . $row->last_name), $row->email, 'auto', '1', array());
				if ($lib_result === false) { // Something went wrong
					return;
				}

				$subId = $lib_result->insertid;
			}

			if ($subId)
			{
				if ($listIds)
				{
					DoRemoveListsFromSubscriber($listIds, $subId);
				}

				if ($assignListIds)
				{
					DoAddListsToSubscriber($assignListIds, $subId);
				}
			}
		}
	}

	/**
	 * Method to synchronize custom fields data from Membership Pro to NS Pro
	 *
	 * @param   OSMembershipTableSubscriber  $row
	 * @param   int                          $subId
	 */
	private function updateNSProFieldValues($row, $subId)
	{
		if (!$subId)
		{
			return;
		}

		$db    = $this->db;
		$query = $db->getQuery(true);

		// Map fields
		$rowFields = OSMembershipHelper::getProfileFields($row->plan_id, true, null, $row->act);
		$data      = OSMembershipHelper::getProfileData($row, $row->plan_id, $rowFields);

		foreach ($rowFields as $rowField)
		{
			if (!$rowField->newsletter_field_mapping)
			{
				continue;
			}

			// Get ID of field, to make sure that it still exists in our fields.
			$query->clear()
				->select('id')
				->from('#__nspro_fields')
				->where('id = ' . $db->quote($rowField->newsletter_field_mapping));
			$db->setQuery($query);
			$fieldId = $db->loadResult();

			if (!$fieldId)
			{
				continue;
			}

			$fieldValue = isset($data[$rowField->name]) ? $data[$rowField->name] : '';

			$query->clear()
				->select('COUNT(*)')
				->from('#__nspro_field_values')
				->where('subscriber_id = ' . $subId)
				->where('field_id = ' . (int) $fieldId);
			$db->setQuery($query);
			$count = (int) $db->loadResult();

			if (!$count)
			{
				$query->clear()
					->insert('#__nspro_field_values')
					->columns($db->quoteName(['subscriber_id', 'field_id', 'value']))
					->values(implode(',', $db->quote([$subId, $fieldId, $fieldValue])));
			}
			else
			{
				$query->update('#__nspro_field_values')
					->set($db->quoteName('value') . '=' . $db->quote($fieldValue))
					->where('subscriber_id = ' . (int) $subId)
					->where('field_id = ' . (int) $fieldId);
			}

			try
			{
				$db->setQuery($query)
					->execute();
			}
			catch (Exception $e)
			{
				// Ignore the error for now
			}
		}
	}

}